@@ -1,16 +1,10 @@
 """Plugin loading and management logic and classes."""
 import logging
-import sys
-
-import entrypoints
+from typing import Any, Dict, List, Optional, Set
 
 from flake8 import exceptions
 from flake8 import utils
-
-if sys.version_info >= (3, 3):
-    import collections.abc as collections_abc
-else:
-    import collections as collections_abc
+from flake8._compat import importlib_metadata
 
 LOG = logging.getLogger(__name__)
 
@@ -37,17 +31,17 @@ class Plugin(object):
         self.name = name
         self.entry_point = entry_point
         self.local = local
-        self._plugin = None
+        self._plugin = None  # type: Any
         self._parameters = None
-        self._parameter_names = None
+        self._parameter_names = None  # type: Optional[List[str]]
         self._group = None
         self._plugin_name = None
         self._version = None
 
-    def __repr__(self):
+    def __repr__(self):  # type: () -> str
         """Provide an easy to read description of the current plugin."""
         return 'Plugin(name="{0}", entry_point="{1}")'.format(
-            self.name, self.entry_point
+            self.name, self.entry_point.value
         )
 
     def to_dictionary(self):
@@ -90,7 +84,7 @@ class Plugin(object):
         return self._parameters
 
     @property
-    def parameter_names(self):
+    def parameter_names(self):  # type: () -> List[str]
         """List of argument names that need to be passed to the plugin."""
         if self._parameter_names is None:
             self._parameter_names = list(self.parameters)
@@ -106,15 +100,15 @@ class Plugin(object):
         return self._plugin
 
     @property
-    def version(self):
+    def version(self):  # type: () -> str
         """Return the version of the plugin."""
-        if self._version is None:
+        version = self._version
+        if version is None:
             if self.is_in_a_group():
-                self._version = version_for(self)
+                version = self._version = version_for(self)
             else:
-                self._version = self.plugin.version
-
-        return self._version
+                version = self._version = self.plugin.version
+        return version
 
     @property
     def plugin_name(self):
@@ -164,7 +158,7 @@ class Plugin(object):
             except Exception as load_exception:
                 LOG.exception(load_exception)
                 failed_to_load = exceptions.FailedToLoadPlugin(
-                    plugin=self, exception=load_exception
+                    plugin_name=self.name, exception=load_exception
                 )
                 LOG.critical(str(failed_to_load))
                 raise failed_to_load
@@ -218,7 +212,8 @@ class Plugin(object):
                 self.name,
                 optmanager,
             )
-            add_options(optmanager)
+            with optmanager.group(self.plugin_name):
+                add_options(optmanager)
 
         if self.off_by_default:
             self.disable(optmanager)
@@ -228,6 +223,7 @@ class PluginManager(object):  # pylint: disable=too-few-public-methods
     """Find and manage plugins consistently."""
 
     def __init__(self, namespace, local_plugins=None):
+        # type: (str, Optional[List[str]]) -> None
         """Initialize the manager.
 
         :param str namespace:
@@ -236,8 +232,8 @@ class PluginManager(object):  # pylint: disable=too-few-public-methods
             Plugins from config (as "X = path.to:Plugin" strings).
         """
         self.namespace = namespace
-        self.plugins = {}
-        self.names = []
+        self.plugins = {}  # type: Dict[str, Plugin]
+        self.names = []  # type: List[str]
         self._load_local_plugins(local_plugins or [])
         self._load_entrypoint_plugins()
 
@@ -250,12 +246,16 @@ class PluginManager(object):  # pylint: disable=too-few-public-methods
         for plugin_str in local_plugins:
             name, _, entry_str = plugin_str.partition("=")
             name, entry_str = name.strip(), entry_str.strip()
-            entry_point = entrypoints.EntryPoint.from_string(entry_str, name)
+            entry_point = importlib_metadata.EntryPoint(name, entry_str, None)
             self._load_plugin_from_entrypoint(entry_point, local=True)
 
     def _load_entrypoint_plugins(self):
         LOG.info('Loading entry-points for "%s".', self.namespace)
-        for entry_point in entrypoints.get_group_all(self.namespace):
+        eps = importlib_metadata.entry_points().get(self.namespace, ())
+        # python2.7 occasionally gives duplicate results due to redundant
+        # `local/lib` -> `../lib` symlink on linux in virtualenvs so we
+        # eliminate duplicates here
+        for entry_point in sorted(frozenset(eps)):
             if entry_point.name == "per-file-ignores":
                 LOG.warning(
                     "flake8-per-file-ignores plugin is incompatible with "
@@ -310,7 +310,7 @@ class PluginManager(object):  # pylint: disable=too-few-public-methods
         :rtype:
             tuple
         """
-        plugins_seen = set()
+        plugins_seen = set()  # type: Set[str]
         for entry_point_name in self.names:
             plugin = self.plugins[entry_point_name]
             plugin_name = plugin.plugin_name
@@ -345,7 +345,7 @@ def version_for(plugin):
 class PluginTypeManager(object):
     """Parent class for most of the specific plugin types."""
 
-    namespace = None
+    namespace = None  # type: str
 
     def __init__(self, local_plugins=None):
         """Initialize the plugin type's manager.
@@ -398,9 +398,7 @@ class PluginTypeManager(object):
     def _generate_call_function(method_name, optmanager, *args, **kwargs):
         def generated_function(plugin):  # noqa: D105
             method = getattr(plugin, method_name, None)
-            if method is not None and isinstance(
-                method, collections_abc.Callable
-            ):
+            if method is not None and callable(method):
                 return method(optmanager, *args, **kwargs)
 
         return generated_function
